==> apache-doris-2.0.15.1-src.tar.gz: apache-doris-2.0.15.1-src/fe/fe-core/src/main/java/org/apache/doris/nereids/rules/rewrite/EliminateNullAwareLeftAntiJoin.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.nereids.rules.rewrite;

import org.apache.doris.nereids.rules.Rule;
import org.apache.doris.nereids.rules.RuleType;
import org.apache.doris.nereids.trees.plans.JoinType;

import java.util.stream.Stream;

/**
 * try to eliminate null aware left anti join
 * if no conjunct is nullable, null aware left anti join can be converted to left anti join safely.
 */
public class EliminateNullAwareLeftAntiJoin extends OneRewriteRuleFactory {
    @Override
    public Rule build() {
        return nullAwareLeftAntiLogicalJoin().then(antiJoin -> {
            if (Stream.concat(antiJoin.getHashJoinConjuncts().stream(), antiJoin.getOtherJoinConjuncts().stream())
                    .noneMatch(expression -> expression.nullable())) {
                return antiJoin.withJoinType(JoinType.LEFT_ANTI_JOIN);
            } else {
                return null;
            }
        }).toRule(RuleType.ELIMINATE_NULL_AWARE_LEFT_ANTI_JOIN);
    }
}

==> apache-doris-2.1.0-src.tar.gz: apache-doris-2.1.0-src/fe/fe-core/src/main/java/org/apache/doris/nereids/rules/rewrite/EliminateNullAwareLeftAntiJoin.java <==
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package org.apache.doris.nereids.rules.rewrite;

import org.apache.doris.nereids.rules.Rule;
import org.apache.doris.nereids.rules.RuleType;
import org.apache.doris.nereids.trees.plans.JoinType;

import java.util.stream.Stream;

/**
 * try to eliminate null aware left anti join
 * if no conjunct is nullable, null aware left anti join can be converted to left anti join safely.
 */
public class EliminateNullAwareLeftAntiJoin extends OneRewriteRuleFactory {
    @Override
    public Rule build() {
        return nullAwareLeftAntiLogicalJoin().then(antiJoin -> {
            if (Stream.concat(Stream.concat(
                            antiJoin.getHashJoinConjuncts().stream(),
                            antiJoin.getOtherJoinConjuncts().stream()),
                            antiJoin.getMarkJoinConjuncts().stream())
                    .noneMatch(expression -> expression.nullable())) {
                return antiJoin.withJoinType(JoinType.LEFT_ANTI_JOIN, antiJoin.getJoinReorderContext());
            } else {
                return null;
            }
        }).toRule(RuleType.ELIMINATE_NULL_AWARE_LEFT_ANTI_JOIN);
    }
}
